"""
<name>Vulcano Plot</name>
<description>Plots fold change vs. p-value.)</description>
<priority>1020</priority>
<contact>Ales Erjavec (ales.erjavec@fri.uni-lj.si)</contact>
<icon>icons/VulcanoPlot.svg</icon>
"""

from __future__ import absolute_import

from operator import add
from collections import defaultdict

import numpy

import orange

from Orange.orng.orngDataCaching import data_hints
from Orange.OrangeWidgets.OWToolbars import ZoomSelectToolbar
from Orange.OrangeWidgets.OWWidget import *
from Orange.OrangeWidgets.OWGraph import *
from Orange.OrangeWidgets import OWGUI

from .. import obiExpression

NAME = "Vulcano Plot"
DESCRIPTION = "Plots fold change vs. p-value.)"
ICON = "icons/VulcanoPlot.svg"
PRIORITY = 1020

INPUTS = [("Examples", Orange.data.Table, "setData")]
OUTPUTS = [("Examples with selected attributes", Orange.data.Table)]

REPLACES = ["_bioinformatics.widgets.OWVulcanoPlot.OWVulcanoPlot"]


class GraphSelections(QObject):
    """ Selection manager using a union of rectangle areas 
    """
    def __init__(self, parent):
        QObject.__init__(self, parent)
        self.selection = []
        
    def getPos(self, event):
        graph = self.parent()
        pos = graph.canvas().mapFrom(graph, event.pos())
        x = graph.invTransform(QwtPlot.xBottom, pos.x())
        y = graph.invTransform(QwtPlot.yLeft, pos.y())
        return QPointF(x, y)
        
    def start(self, event):
        pos = self.getPos(event)
        if event.modifiers() & Qt.ControlModifier:
            self.selection.append((pos, pos))
        else:
            self.selection = [(pos, pos)]
        self.emit(SIGNAL("selectionGeometryChanged()"))
    
    def update(self, event):
        pos = self.getPos(event)
        self.selection[-1] = self.selection[-1][:-1] + (pos,)
        self.emit(SIGNAL("selectionGeometryChanged()"))
    
    def end(self, event):
        self.update(event)
        
    def testSelection(self, data):
        if len(data) == 0:
            return []
        data = numpy.asarray(data)
        region = QPainterPath()
        for p1, p2 in self.selection:
            region.addRect(QRectF(p1, p2).normalized())
        def test(point):
            return region.contains(QPointF(point[0], point[1]))
        test = numpy.apply_along_axis(test, 1, data)
        return test
        
class SymetricSelections(GraphSelections):
    """ Selection manager using two symmetric areas extending to 'infinity'  
    """
    def __init__(self, parent, x=3, y=3):
        GraphSelections.__init__(self, parent)
        max = 100000
        self.selection = [(QPointF(-max, max), QPointF(-x, y)), (QPointF(max, max), QPointF(x, y))]
        self.updateAxes = None
        
    def updateSelection(self, axes, pos):
        if axes == QwtPlot.xBottom or axes == -1:
            self.selection[0][1].setX(-abs(pos.x()))
            self.selection[1][1].setX(abs(pos.x()))
        if axes == QwtPlot.yLeft or axes == -1:
            self.selection[0][1].setY(pos.y())
            self.selection[1][1].setY(pos.y())
            
        self.emit(SIGNAL("selectionGeometryChanged()"))
        
    def getAxesAndPos(self, event):
        graph = self.parent()
        pos = graph.canvas().mapFrom(graph, event.pos())
        x = graph.invTransform(QwtPlot.xBottom, pos.x())
        y = graph.invTransform(QwtPlot.yLeft, pos.y())
        
        offset = 3
        dx = abs(graph.invTransform(QwtPlot.xBottom, pos.x() + offset) - x)
        dy = abs(graph.invTransform(QwtPlot.yLeft, pos.y() + offset) - y)
        
        x = abs(x)
        
        cx = self.selection[1][1].x()
        cy = self.selection[1][1].y()

        bottom = QRectF(QPointF(cx, cy), QPointF(graph.maxX, cy)).adjusted(-dx, dy, dx, -dy).normalized()
        left = QRectF(QPointF(cx, graph.maxY), QPointF(cx, cy)).adjusted(-dx, dy, dx, -dy).normalized()
        
        if bottom.contains(QPointF(x, y)) or bottom.contains(QPointF(-x, y)):
            axes = QwtPlot.yLeft
        elif left.contains(QPointF(x, y)) or left.contains(QPointF(-x, y)):
            axes = QwtPlot.xBottom
        else:
            axes = -1
        return axes, QPointF(x, y)
        
    def start(self, event):
        axes, pos = self.getAxesAndPos(event)
        self.updateAxes = axes
        self.updateSelection(axes, pos)
        
    def update(self, event):
        _, pos = self.getAxesAndPos(event)
        self.updateSelection(self.updateAxes, pos)
    
    def end(self, event):
        self.update(event)
        self.updateAxes = None
        
    def testSelection(self, data):
        if len(data) == 0:
            return []
        data = numpy.asarray(data)
        cutoffX = self.selection[1][1].x()
        cutoffY = self.selection[1][1].y()
        return (numpy.abs(data[:, 0]) >= cutoffX) & (data[:, 1] >= cutoffY)

    def cuttoff(self):
        """
        Return the absolute x, y cutoff values.
        """
        return (self.selection[1][1].x(), self.selection[1][1].y())

from Orange.OrangeWidgets.OWItemModels import PyListModel


def item_selection(indices, model, selection=None, column=0):
    """ Create an QItemSelection for indices in model.
    """
    if selection is None:
        selection = QItemSelection()
        
    for i in indices:
        selection.select(model.index(i, column))
    return selection


class LabelSelectionWidget(QWidget):
    """ A widget for selection of label values.
    """
    def __init__(self, parent=None):
        QWidget.__init__(self, parent)
        self._values_model = PyListModel([], parent=self)
        layout = QVBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        def group_box(title):
            box = QGroupBox(title)
            box.setFlat(True)
            lay = QVBoxLayout()
            lay.setContentsMargins(0, 0, 0, 0)
            box.setLayout(lay)
            return box
        
        self.labels_combo = QComboBox()
        self.values_view = QListView()
        self.values_view.setSelectionMode(QListView.ExtendedSelection)
        self.values_view.setModel(self._values_model)
        
        
        self.connect(self.labels_combo, SIGNAL("activated(int)"),
                     self.on_label_activated)
        
        self.connect(self.values_view.selectionModel(),
                     SIGNAL("selectionChanged(QItemSelection, QItemSelection)"),
                     self.on_values_selection)
        
        l_box = group_box("Label")
        v_box = group_box("Values")
        
        l_box.layout().addWidget(self.labels_combo)
        v_box.layout().addWidget(self.values_view)
        
        layout.addWidget(l_box)
        layout.addWidget(v_box)
        
        self.setLayout(layout)
        
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        
        self._block_selection_emit = False
        
        self.labels = []
        self.set_labels([])
        
    def clear(self):
        self.labels_combo.clear()
        self._values_model[:] = []
        self.labels = []
        
    def set_labels(self, labels):
        """ Set the labels to display.
        """
        self.clear()
        if isinstance(labels, dict):
            labels = labels.items()
            
        self.labels = labels
        for label, values in labels:
            self.labels_combo.addItem(label)
            
        if labels:
            self.set_current_label(0)
#            self.labels_combo.setCurrentIndex(0)
            
    def set_selection(self, label, values):
        """ Set the selection to label and values
        """
        if isinstance(label, basestring):
            labels = [l for l, _ in self.labels]
            index = labels.index(label) if label in labels else -1
        else:
            index = label
            
        if index >= 0:
            if index != self.labels_combo.currentIndex():
                self.set_current_label(index)
                
            all_values = list(self._values_model)
            values = [v for v in values if v in all_values]
            selection = QItemSelection()
            for i, v in enumerate(self._values_model):
                if v in values:
                    index = self._values_model.index(i, 0)
                    selection.select(index, index)
            self.values_view.selectionModel().select(selection,  QItemSelectionModel.ClearAndSelect)
        else:
            self.values_view.selectionModel().clear()
            
    def set_current_label(self, index):
        """ Set the current label
        """
        self.labels_combo.setCurrentIndex(index)
        label, values = self.labels[index]
        # Block selection changed
        with self._blocked_signals():
            self._values_model[:] = values
        
    def on_label_activated(self, index):
        label, values = self.labels[index]
        with self._blocked_signals():
            self._values_model[:] = values
        self.emit(SIGNAL("label_activated()"))
        self.emit(SIGNAL("label_activated(int)"), index)
    
    def on_values_selection(self, selected, deselected):
        label, values = self.current_selection()
        self.emit(SIGNAL("selection_changed()"))
        self.emit(SIGNAL("selection_changed(PyQt_PyObject, PyQt_PyObject)"),
                  label, values)
        
    def selection_indexes(self):
        """ Return the values selection indices.
        """
        selection = self.values_view.selectionModel().selection()
        indexes = selection.indexes()
        return sorted(set([i.row() for i in indexes]))        
        
    def current_selection(self):
        """ Return the current label and selected values.
        """
        i = self.labels_combo.currentIndex()

        if i == -1:
            # When clearing the labels model / combobox
            return None, None

        label, all_values = self.labels[i]
        values = [all_values[i] for i in self.selection_indexes()]
        return label, values
    
    def _blocked_signals(self):
        """ Return a context handler blocking all emited signals from this
        object.
         
        """
        class block(object):
            def __enter__(blocker):
                self.blockSignals(True)
            def __exit__(blocker, *args):
                self.blockSignals(False)
                return False
        return block()
                
    def sizeHint(self):
        return QSize(100, 200)


class VulcanoGraph(OWGraph):
    def __init__(self, *args, **kwargs):
        OWGraph.__init__(self, *args, **kwargs)
        # Absolute cutoff values for symmetric selection mode.
        self.cutoffX = 2.0
        self.cutoffY = 2.0
        # maximum absolute x, y values.
        self.maxX, self.maxY = 10, 10
        self.symbolSize = 5
        self.symetricSelections = True

        self.selectedCurve = self.addCurve("", brushColor=Qt.red)
        self.unselectedCurve = self.addCurve("", brushColor=Qt.blue)

        self.plotValues = {}

        self.setAxisAutoScale(QwtPlot.xBottom)
        self.setAxisAutoScale(QwtPlot.yLeft)

        self.setSelection(
            SymetricSelections(
                self,
                x=min(self.maxX, self.cutoffX),
                y=min(self.maxY, self.cutoffY))
        )

    def setSelection(self, selection):
        self.selection = selection
        self.connect(self.selection, SIGNAL("selectionGeometryChanged()"), self.onSelectionChanged)
        if self.plotValues:
            self.updateSelectionArea()

    def onSelectionChanged(self):
        if self.symetricSelections:
            self.cutoffX, self.cutoffY = self.selection.cuttoff()

        self.replot_()
        self.emit(SIGNAL("selectionChanged()"))

    def splitSelected(self):
        test =  self.selection.testSelection(self.plotData)
        return (self.plotData[numpy.nonzero(test)], self.plotData[numpy.nonzero(~test)])

    def setPlotValues(self, values):
        self.plotValues = values
        self.plotData = numpy.array(values.values())
        if self.plotData.size:
            self.maxX = numpy.max(numpy.abs(self.plotData[:, 0]))
            self.maxY = numpy.max(self.plotData[:, 1])
        else:
            self.maxX, self.maxY = 10, 10

        self.setAxisScale(QwtPlot.xBottom, -self.maxX, self.maxX)
        self.setAxisScale(QwtPlot.yLeft, 0.0, self.maxY)
        self.replot_()
        self.emit(SIGNAL("selectionChanged()"))

    def createSelectionRectCurve(self, p1, p2):
        curve = self.addCurve("selection", style=QwtPlotCurve.Lines, penColor=Qt.red, symbol=QwtSymbol.NoSymbol)
        curve.setData([p1.x(), p2.x(), p2.x(), p1.x(), p1.x()], [p1.y(), p1.y(), p2.y(), p2.y(), p1.y()])
        
    def items(self, title=None):
        for item in self.itemList():
            if str(item.title().text()) == title:
                yield item
        
    def updateSelectionArea(self):
        for c in self.items(title="selection"):
            c.detach()
        for p1, p2 in self.selection.selection:
            self.createSelectionRectCurve(p1, p2)

    def replot_(self):
        if self.plotValues:
            selected, unselected = self.splitSelected()
            self.selectedCurve.setData(selected[:, 0], selected[:, 1])
            self.unselectedCurve.setData(unselected[:, 0], unselected[:, 1])
            self.updateSelectionArea()
        else:
            for curve in [self.selectedCurve, self.unselectedCurve]:
                curve.setData([], [])
        self.replot()

    def mousePressEvent(self, event):
        canvas = self.canvas()
        canvasPos = canvas.mapFrom(self, event.pos())
        if canvas.contentsRect().contains(canvasPos) and \
                self.state == SELECT and event.button() == Qt.LeftButton:
            self.selection.start(event)
        else:
            OWGraph.mousePressEvent(self, event)

    def mouseMoveEvent(self, event):
        if self.state == SELECT:
            if event.buttons() & Qt.LeftButton:
                self.selection.update(event)
            if isinstance(self.selection, SymetricSelections):
                axes, pos = self.selection.getAxesAndPos(event)
                cursors = {QwtPlot.xBottom: Qt.SizeHorCursor,
                           QwtPlot.yLeft: Qt.SizeVerCursor}
                self.canvas().setCursor(cursors.get(axes, self._cursor))
        else:
            OWGraph.mouseMoveEvent(self, event)

    def mouseReleaseEvent(self, event):
        if self.state == SELECT:
            if event.button() == Qt.LeftButton:
                self.selection.end(event)
        else:
            OWGraph.mouseReleaseEvent(self, event)

    def reselect(self, replot=True):
        if self.symetricSelections:
            self.setSelection(
                SymetricSelections(
                    self,
                    x=min(self.maxX, self.cutoffX),
                    y=min(self.maxY, self.cutoffY))
            )
        else:
            self.setSelection(GraphSelections(self))
            self.canvas().setCursor(self._cursor)
        if replot:
            self.replot_()

    def updateSymbolSize(self):
        def setSize(curve, size):
            symbol = curve.symbol()
            symbol.setSize(size)
            if QWT_VERSION_STR >= "5.2":
                curve.setSymbol(symbol)
        setSize(self.selectedCurve, self.symbolSize)
        setSize(self.unselectedCurve, self.symbolSize)
        self.replot()


from .OWGenotypeDistances import SetContextHandler
from .OWFeatureSelection import disable_controls


class OWVulcanoPlot(OWWidget):
    settingsList = ["graph.cutoffX", "graph.cutoffY", "graph.symbolSize",
                    "graph.symetricSelections", "showXTitle", "showYTitle"]

    contextHandlers = {
        "targets": SetContextHandler("targets")
    }

    def __init__(self, parent=None, signalManager=None, name="Vulcano Plot"):
        OWWidget.__init__(self, parent, signalManager, name, wantGraph=True)

        self.inputs = [("Examples", ExampleTable, self.setData)]
        self.outputs = [("Examples with selected attributes", ExampleTable)]

        self.genes_in_columns = False

        self.showXTitle = True
        self.showYTitle = True

        self.auto_commit = False
        self.selection_changed_flag = False
        self.target_group = None, []
        self.label_selections = []

        self.graph = VulcanoGraph(self)
        self.connect(self.graph, SIGNAL("selectionChanged()"),
                     self.on_selection_changed)
        self.mainArea.layout().addWidget(self.graph)

        self.loadSettings()

        ## GUI
        box = OWGUI.widgetBox(self.controlArea, "Info")
        self.infoLabel = OWGUI.label(box, self, "")
        self.infoLabel.setText("No data on input")
        self.infoLabel2 = OWGUI.label(box, self, "")
        self.infoLabel2.setText("0 selected genes")
        
        box = OWGUI.widgetBox(self.controlArea, "Target Labels")
        
        self.target_widget = LabelSelectionWidget(self)
        self.connect(self.target_widget,
                     SIGNAL("selection_changed(PyQt_PyObject, PyQt_PyObject)"),
                     self.on_target_changed)
        self.connect(self.target_widget,
                     SIGNAL("label_activated(int)"),
                     self.on_label_activated)
        
        box.layout().addWidget(self.target_widget)
        
        self.genesInColumnsCheck = OWGUI.checkBox(box, self, "genes_in_columns",
                                    "Genes in columns", 
                                    callback=[self.init_from_data, self.plot])

        box = OWGUI.widgetBox(self.controlArea, "Settings")
        OWGUI.hSlider(box, self, "graph.symbolSize", label="Symbol size:   ", minValue=2, maxValue=20, step=1, callback = self.graph.updateSymbolSize)
        OWGUI.checkBox(box, self, "showXTitle", "X axis title", callback=self.setAxesTitles)
        OWGUI.checkBox(box, self, "showYTitle", "Y axis title", callback=self.setAxesTitles)
        
        toolbar = ZoomSelectToolbar(self, self.controlArea, self.graph, buttons=[ZoomSelectToolbar.IconSelect, ZoomSelectToolbar.IconZoom, ZoomSelectToolbar.IconPan])
        
        top_layout = toolbar.layout()
        top_layout.setDirection(QBoxLayout.TopToBottom)
        button_layotu = QHBoxLayout()
        top_layout.insertLayout(0, button_layotu)
        
        for i in range(1, top_layout.count()):
            item = top_layout.itemAt(1)
            top_layout.removeItem(item)
            button_layotu.addItem(item)
        
        OWGUI.checkBox(toolbar, self, "graph.symetricSelections", "Symetric selection", callback=self.graph.reselect)

        box = OWGUI.widgetBox(self.controlArea, "Commit")
        b = OWGUI.button(box, self, "Commit", callback=self.commit, default=True)
        cb = OWGUI.checkBox(box, self, "auto_commit", "Commit automatically")
        OWGUI.setStopper(self, b, cb, "selection_changed_flag", self.commit_if)

        self.connect(self.graphButton, SIGNAL("clicked()"), self.graph.saveToFile)
        
        OWGUI.rubber(self.controlArea)

        self.data = None
        self.target_group = None, []
        self.current_selection = []

        self.graph.reselect(True)
        self.resize(800, 600)

    def clear(self):
        self.target_widget.set_labels([])
        self.targets = []
        self.label_selections = []
        self.target_group = None, []
        self.clear_graph()
        
    def clear_graph(self):
        self.values = {}
        self.graph.setPlotValues({})
        self.updateTooltips()

    def setData(self, data=None):
#         self.closeContext("")
        self.closeContext("targets")
        self.clear()
        self.data = data
        self.error(0)
        self.warning([0, 1])
        if data:
            self.genes_in_columns = not bool(data.domain.classVar)
            self.genesInColumnsCheck.setDisabled(not bool(data.domain.classVar))
            if self.genes_in_columns:
                self.genes_in_columns = not data_hints.get_hint(data, "genesinrows", not self.genes_in_columns)
#             self.openContext("", data)
        else:
            self.infoLabel.setText("No data on input.")
        self.init_from_data()

    def init_from_data(self):
        """ Init widget state from the data.
        """
        self.update_target_labels()
        self.error(0)
        if self.data:
            if not self.targets:
                if self.genes_in_columns:
                    self.error(0, "Data set with no column labels (attribute tags)")
                else:
                    self.error(0, "Data has no class.")
        
        self.openContext("targets", [(label, v) for label, vals in self.targets \
                                                for v in vals])
        
        if len(self.label_selections) != len(self.targets): # Some times this happens.
            self.label_selections = [[] for t in self.targets]
            
        if self.target_group == (None, []) and self.targets:
            label, values = self.targets[0]
            self.target_group = (label, values[:1])
            
        if self.target_group != (None, []):
            self.target_widget.set_selection(*self.target_group)
        else:
            self.clear_graph()

    def update_target_labels(self):
        if self.data:
            if self.genes_in_columns:
                items = [a.attributes.items() for a in self.data.domain.attributes]
                items = reduce(add, items, [])
                
                targets = defaultdict(set)
                for label, value in items:
                    targets[label].add(value)
                    
                targets = [(key, list(sorted(vals))) for key, vals in targets.items() \
                           if len(vals) >= 2]
                self.targets = targets
                
            else:
                var = self.data.domain.classVar
                values = list(var.values)
                if len(values) >= 2:
                    self.targets = [(var.name, values)]
                else:
                    self.targets = []
        else:
            self.targets = []
            
        if self.targets:
            label, values = self.targets[0]
            self.target_group = (label, values[:1])
        else:
            self.target_group = None, []
            
        self.label_selections = [[] for t in self.targets]
        self.target_widget.set_labels(self.targets)
                
        
    def on_label_activated(self, index):
        """ Try to restore a saved selection.
        """
        selected = self.label_selections[index]
        if not selected:
            selected = self.targets[index][1][:1]
            
        self.target_widget.set_selection(index, selected)
        
    def on_target_changed(self, label, values):
        self.target_group = label, values
        # Save the selection
        labels = [l for l, _ in self.targets]
        if label in labels:
            index = labels.index(label)
            self.label_selections[index] = values
            
        # replot
        if label and values:
            self.plot()
        else:
            self.clear_graph()
    
    @disable_controls
    def plot(self):
        self.values = {}
        self.current_selection = []
        target_label, target_values = self.target_group
        self.warning([0, 1])
        self.error(1)
        if self.data and target_values:
            target_label, target_values = self.target_group
            if self.genes_in_columns:
                target = set([(target_label, value) for value in target_values])
            else:
                target = set(target_values)
            
            ttest = obiExpression.ExpressionSignificance_TTest(self.data, useAttributeLabels=self.genes_in_columns)
            ind1, ind2 = ttest.test_indices(target)
            
            if not len(ind1) or not len(ind2):
                self.error(1, "Target labels most exclude/include at least one value.")
                
            if len(ind1) < 2 and len(ind2) < 2:
                self.warning(0, "Insufficient data to compute statistics. More than one measurement per class should be provided")
            
            self.progressBarInit()
            try:
                tt = ttest(target)
                self.progressBarSet(25)
                fold = obiExpression.ExpressionSignificance_FoldChange(self.data, useAttributeLabels=self.genes_in_columns)(target)
                self.progressBarSet(50)
            except ZeroDivisionError, ex:
                tt, fold = [], []
            self.infoLabel.setText("%i genes on input" % len(fold))
            
            invalid = set([key for (key, (t, p)), (_, f) in zip(tt, fold) if any(v is numpy.ma.masked for v in [t, p, f]) or f==0.0])
            tt = [t for t in tt if t[0] not in invalid]
            fold = [f for f in fold if f[0] not in invalid]
            self.progressBarSet(75)
            logratio = numpy.log2(numpy.abs([v for k, v in fold]))
            logpval = -numpy.log10([p for k, (t, p) in tt])
            self.values = dict(zip([k for k, v in tt], zip(logratio, logpval)))
            if not self.values:
                self.warning(1, "Could not compute statistics for any genes!")
            self.progressBarFinished()
        self.graph.setPlotValues(self.values)
        self.setAxesTitles()
        self.updateTooltips()

    def setAxesTitles(self):
        self.graph.setAxisTitle(QwtPlot.xBottom, "log<sub>2</sub> (ratio)" if self.showXTitle else "")
        self.graph.setAxisTitle(QwtPlot.yLeft, "-log<sub>10</sub> (p_value)" if self.showYTitle else "")

    def updateTooltips(self):
        self.graph.tips.removeAll()
        for key, (logratio, logpval) in self.values.items():
            self.graph.tips.addToolTip(logratio, logpval, "<b>%s</b><hr>log<sub>2</sub>(ratio): %.5f<br>p-value: %.5f" \
                                       %(str(key) if self.genes_in_columns else key.name, logratio, math.pow(10, -logpval)))

    def selection(self, items=None):
        """
        Return the current selection.
        """
        if items is None:
            items = sorted(self.values.items())
        values = [val for key, val in items]
        test = self.graph.selection.testSelection(values)
        return test

    def on_selection_changed(self):
        # Selection area on the plot has changed.
        nselected = self.graph.selectedCurve.data().size()
        self.infoLabel2.setText("%i selected genes" % nselected)

        if self.auto_commit:
            selection = list(self.selection())
            # Did the selection actually change
            if selection != self.current_selection:
                self.current_selection = selection
                self.commit()
        else:
            self.selection_changed_flag = True
    
    def commit(self):
        if self.data and self.genes_in_columns:
            items = sorted(self.values.items())
            test = self.selection(items)
            selected = [self.data[i] for t, (i, value) in zip(test, items) if t]
            if selected:
                data = orange.ExampleTable(self.data.domain, selected)
            else:
                data = None
            self.current_selection = list(test) # For testing in on_selection_changed
        elif self.data:
            attrs = [(attr, self.values[attr])  for attr in self.data.domain.attributes if attr in self.values]
            test = self.selection(attrs)
#            test = self.graph.selection.testSelection([val for attr, val in attrs])
            selected = [attr for t, (attr, val) in zip(test, attrs) if t]
            newdomain = orange.Domain(selected + [self.data.domain.classVar])
            newdomain.addmetas(self.data.domain.getmetas())
            data = orange.ExampleTable(newdomain, self.data)
            self.current_selection = list(test) # For testing in on_selection_changed
        else:
            data = None
        
        self.send("Examples with selected attributes", data)
        self.selection_changed_flag = False

    def commit_if(self):
        if self.auto_commit:
            self.commit()
        else:
            self.selection_changed_flag = True
            
    def settingsToWidgetCallbacktargets(self, handler, context):
        self.label_selections = list(getattr(context, "label_selections", self.label_selections))
        self.target_group = getattr(context, "target_group", self.target_group)
        
    def settingsFromWidgetCallbacktargets(self, handler, context):
        context.label_selections = list(self.label_selections)
        context.target_group = self.target_group


if __name__ == "__main__":
    ap = QApplication(sys.argv)
    w = OWVulcanoPlot()
    d = orange.ExampleTable("brown-selected.tab")
    w.setData(d)
    w.show()
    ap.exec_()
    w.saveSettings()



## Automatically adapted for numpy.oldnumeric Oct 04, 2007 by 

"""
<name>Data Distance</name>
<description>Computes a distance matrix between data files.</description>
<icon>icons/ChipDistance.png</icon>
<priority>1160</priority>
<contact>Peter Juvan (peter.juvan@fri.uni-lj.si)</contact>
<prototype>1</prototype>
"""

from __future__ import absolute_import

from qt import *
from qtcanvas import *

import numpy.oldnumeric as Numeric, numpy.oldnumeric.ma as MA

import orange, statc
from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWWidget import *

from .OWDataFiles import DataFiles

import warnings
warnings.filterwarnings("ignore", "'strain'", orange.AttributeWarning)
warnings.filterwarnings("ignore", "'dirname'", orange.AttributeWarning)

##############################################################################
# main class

class OWDataDistance(OWWidget):
    settingsList = ["Metrics"]

    def __init__(self, parent=None, signalManager = None):
        OWWidget.__init__(self, parent, signalManager, 'Data Distance') 
        
        self.inputs = [("Structured Data", DataFiles, self.chipdata)]
        self.outputs = [("Distance Matrix", orange.SymMatrix)]

        self.Metrics = 0
        self.loadSettings()
        self.data = []
##        self.metrics = [("Euclidean", orange.ExamplesDistanceConstructor_Euclidean),
##            ("Manhattan", orange.ExamplesDistanceConstructor_Manhattan),
##            ("Hamming", orange.ExamplesDistanceConstructor_Hamming)]
        self.metrics = [("Manhattan", distManhattan), ("Euclidean", distEuclidean), ("1 - (Pearson correlation coefficient)", distPearson), ("1 - (Spearman rank correlation coefficient)", distSpearman)]

        # GUI
        self.mainArea.setFixedWidth(0)
        # Info box
        box = QVGroupBox("Info", self.controlArea)
        self.infoa = QLabel('No data on input.', box)
        self.infob = QLabel('', box)
        OWGUI.separator(self.controlArea)

        # Distance metrics selection
        items = [x[0] for x in self.metrics]
        OWGUI.comboBox(self.controlArea, self, "Metrics", box="Distance Metrics", items=items,
            tooltip="Metrics to measure distance between data sets.",
            callback=self.onMetricsChange)

        self.resize(384, 138)
        

    ##########################################################################
    # handling of input/output signals

##    def computeDistance(self, d1, d2, dist):
##        """employs orange to compute distances (slower)
##        """
##        d = 0
##        for i in range(len(d1)):
##            d += dist(d1[i], d2[i])
##        d = d / len(d1)
##        return d

    def computeDistance(self, d1, d2):
        """employs MA to cumpute distances (faster)
        """
        return dist(d1.toNumpyMA("a")[0], d2.toNumpyMA("a")[0])


    def computeMatrix(self):
        if not self.data:
            self.send("Distance Matrix", None)
            return
##        if self.Metrics == 0: # bug in orange, correct (remove normalize) once it is fixed
##            dist = self.metrics[self.Metrics][1](self.data[0], normalize=0)
##        else:
##            dist = self.metrics[self.Metrics][1](self.data[0])            
        matrix = orange.SymMatrix(len(self.data))
        matrix.setattr('items', self.data)
        self.progressBarInit()
        pbStep = 100./(len(self.data)**2/2. - len(self.data)/2.)
        for i in range(len(self.data)-1):
            for j in range(i+1, len(self.data)):
##                matrix[i, j] = self.computeDistance(self.data[i], self.data[j], dist)
                matrix[i, j] = self.metrics[self.Metrics][1](MA.ravel(self.data[i].toNumpyMA("a")[0]), MA.ravel(self.data[j].toNumpyMA("a")[0]))
                self.progressBarAdvance(pbStep)
        self.progressBarFinished()
        self.send("Distance Matrix", matrix)


    def chipdata(self, data):
        self.data = []
        if data:
            self.infob.setText("")
            numFiles = reduce(lambda a,b: a+len(b[1]), data, 0)
            lenSD = len(data)
            self.infoa.setText("%d set%s, total of %d data file%s." % (lenSD, ["","s"][lenSD!=1], numFiles, ["","s"][numFiles!=1]))
            numExamplesList = []
            # construct a list of ExampleTable lengths and a list of attribute names
            for (name, etList) in data:
                for et in etList:
                    setattr(et,"dirname",name)
                    setattr(et,"strain",name)
                    self.data.append(et)
                    numExamplesList.append(len(et))
            if len(self.data)>1:
                # test that files contain the same attributes and equal number of examples
                attrSorted = self.data[0].domain.attributes
                attrSorted.sort()
                numEx = len(self.data[0])
                for et in self.data[1:]:
                    attrSorted2 = et.domain.attributes
                    attrSorted2.sort()
                    if map(lambda x: x.name, attrSorted) != map(lambda x: x.name, attrSorted2):
                        self.data = []
                        self.infob.setText("Error: data files contain different attributes, aborting distance computation.")
                        return
                    if len(et) != numEx:
                        self.data = []
                        self.infob.setText("Error: data files contain unequal number of examples, aborting distance computation.")
                        return
                # compute distances
                pb = OWGUI.ProgressBar(self, iterations=len(self.data))
                self.computeMatrix()
                pb.finish()

            else:
                self.data = []
                self.infob.setText('Error: not enough data, aborting distance computation.')
        else:
            self.infoa.setText('No data on input.')


    def onMetricsChange(self):
        if self.data and len(self.data)>1:
            self.computeMatrix()



###########################################################################
# Distance Metrics
###########################################################################

def distManhattan(x,y):
    """normalized Manhattan distance
    """
    x = MA.asarray(x)
    y = MA.asarray(y)
    assert MA.rank(x) == MA.rank(y) == 1
    sumWeights = MA.add.reduce(MA.logical_not(MA.logical_or(MA.getmaskarray(x), MA.getmaskarray(y))).astype(Numeric.Float))
    return MA.add.reduce(MA.absolute(x-y)) / sumWeights


def distManhattanW(x,y,w):
    """normalized weighted Manhattan distance
    """
    x = MA.asarray(x)
    y = MA.asarray(y)
    w = MA.asarray(w)
    assert MA.rank(x) == MA.rank(y) == MA.rank(w) == 1
    sumWeights = MA.add.reduce(w * MA.logical_not(MA.logical_or(MA.getmaskarray(x), MA.getmaskarray(y))).astype(Numeric.Float))
    return MA.add.reduce(w * MA.absolute(x-y)) / sumWeights


def distEuclidean(x,y):
    """normalized euclidean distance
    """
    x = MA.asarray(x)
    y = MA.asarray(y)
    assert MA.rank(x) == MA.rank(y) == 1
    sumWeights = MA.add.reduce(MA.logical_not(MA.logical_or(MA.getmaskarray(x), MA.getmaskarray(y))).astype(Numeric.Float))
    return MA.sqrt(MA.add.reduce((x-y)**2) / sumWeights)


def distEuclideanW(x,y,w):
    """normalized weighted euclidean distance
    """
    x = MA.asarray(x)
    y = MA.asarray(y)
    w = MA.asarray(w)
    assert MA.rank(x) == MA.rank(y) == MA.rank(w) == 1
    sumWeights = MA.add.reduce(w * MA.logical_not(MA.logical_or(MA.getmaskarray(x), MA.getmaskarray(y))).astype(Numeric.Float))
    return MA.sqrt(MA.add.reduce(w * (x-y)**2) / sumWeights)


def distPearson(x,y):
    """distance corresponding to 1 - pearson's correlation coefficient for arrays x,y
    returns distance: 1 - pearson_r
    """
    x = MA.asarray(x)
    y = MA.asarray(y)
    assert MA.rank(x) == MA.rank(y) == 1
    cond = MA.logical_not(MA.logical_or(MA.getmaskarray(x), MA.getmaskarray(y)))
    return 1 - statc.pearsonr(MA.compress(cond,x).tolist(), MA.compress(cond,y).tolist())[0]


def distPearsonW(x,y,w):
    """weighted distance corresponding to 1 - pearson's correlation coefficient for arrays x,y and weights w
    returns distance: 1 - pearson_r
    """
    #TINY = 1.0e-20
    # ones for non-masked places at x,y and w
    x = MA.asarray(x)
    y = MA.asarray(y)
    w = MA.asarray(w)
    assert MA.rank(x) == MA.rank(y) == MA.rank(w) == 1
    mask = MA.logical_or(MA.logical_or(MA.getmaskarray(x), MA.getmaskarray(y)), MA.getmaskarray(w))
    # set mask to w that is equal to the mask from x, y and w
    w = MA.masked_array(w, mask=mask)
    n_w_mean = MA.add.reduce(w)    # n * mean(w)
    x_w = x*w       # x * w
    y_w = y*w       # y * w
    x_wmean = MA.divide(MA.add.reduce(x_w), n_w_mean)     # weighted_mean(x)
    y_wmean = MA.divide(MA.add.reduce(y_w), n_w_mean)     # weighted_mean(x)    
    r_num = MA.add.reduce(x*y*w) - n_w_mean*x_wmean*y_wmean
    r_den = MA.sqrt((MA.add.reduce(x_w*x) - n_w_mean*x_wmean**2) * (MA.add.reduce(y_w*y) - n_w_mean*y_wmean**2))
    return 1 - MA.divide(r_num, r_den)


def distSpearman(x,y):
    """distance corresponding to 1 - spearman's correlation coefficient for arrays x,y
    returns distance: 1 - spearman_r
    """
    x = MA.asarray(x)
    y = MA.asarray(y)
    assert MA.rank(x) == MA.rank(y) == 1
    cond = MA.logical_not(MA.logical_or(MA.getmaskarray(x), MA.getmaskarray(y)))
    return 1 - statc.spearmanr(MA.compress(cond,x).tolist(), MA.compress(cond,y).tolist())[0]

def distSpearmanW(x,y,w):
    """weighted distance corresponding to 1 - spearman's correlation coefficient for arrays x,y and weights w
    returns distance: 1 - spearman_r
    """
    distSpearFunc = _distSpearmanW_NU
    for var in (x,y,w):
        if type(var) == MA.array and MA.count(var) != Numeric.multiply.reduce(var.shape):
            distSpearFunc = _distSpearmanW_MA
            break
    return distSpearFunc(x,y,w)


def _distSpearmanW_NU(x,y,w):
    """x,y,w must be Numeric
    """
    x = Numeric.asarray(x)
    y = Numeric.asarray(y)
    w = Numeric.asarray(w)
    assert Numeric.rank(x) == Numeric.rank(y) == Numeric.rank(w) == 1
    rankx = Numeric.array(statc.rankdata(x.tolist()))
    ranky = Numeric.array(statc.rankdata(y.tolist()))
    return distPearsonW(rankx,ranky,w)


def _distSpearmanW_MA(x,y,w):
    """if any of x,y,w is a MA array containing masked values
    """
    x = MA.asarray(x)
    y = MA.asarray(y)
    w = MA.asarray(w)
    assert MA.rank(x) == MA.rank(y) == MA.rank(w) == 1
    cond = MA.logical_not(MA.logical_or(MA.logical_or(MA.getmaskarray(x), MA.getmaskarray(y)), MA.getmaskarray(w))) 
    # with MA use compress before tolist() !
    rankx = Numeric.array(statc.rankdata(MA.compress(cond, x).tolist()))
    ranky = Numeric.array(statc.rankdata(MA.compress(cond, y).tolist()))
    return distPearsonW(rankx,ranky,MA.compress(cond,w))

###########################################################################
# testing
###########################################################################

if __name__=="__main__":
    from . import OWDataFiles
    from Orange.orng import orngSignalManager
    signalManager = orngSignalManager.SignalManager(0)
    a=QApplication(sys.argv)
    ow=OWDataDistance(signalManager = signalManager)
    signalManager.addWidget(ow)
    a.setMainWidget(ow)
    ow.show()
    ds = OWDataFiles.OWDataFiles(signalManager = signalManager)
    signalManager.addWidget(ds)
    ds.loadData("potato.sub100")
    signalManager.setFreeze(1)
    signalManager.addLink(ds, ow, 'Structured Data', 'Structured Data', 1)
    signalManager.setFreeze(0)
    a.exec_loop()
    ow.saveSettings()



#!/usr/bin/env python

try:
    import distribute_setup
    distribute_setup.use_setuptools()
except ImportError:
    # For documentation we load setup.py to get version
    # so it does not matter if importing fails
    pass

import os

from setuptools import setup, find_packages

NAME = 'Orange-Bioinformatics'
DOCUMENTATION_NAME = 'Orange Bioinformatics'

VERSION = '2.5.33'

DESCRIPTION = 'Orange Bioinformatics add-on for Orange data mining software package.'
LONG_DESCRIPTION = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
AUTHOR = 'Bioinformatics Laboratory, FRI UL'
AUTHOR_EMAIL = 'contact@orange.biolab.si'
URL = 'http://orange.biolab.si/addons/'
DOWNLOAD_URL = 'https://bitbucket.org/biolab/orange-bioinformatics/downloads'
LICENSE = 'GPLv3'

KEYWORDS = (
    'data mining',
    'machine learning',
    'artificial intelligence',
    'bioinformatics',
    'gene ontology',
    'KEGG',
    'expression profiles',
    'microarray',
    'genomics',
    'orange',
    'orange add-on',
)

CLASSIFIERS = (
    'Development Status :: 4 - Beta',
    'Environment :: X11 Applications :: Qt',
    'Environment :: Console',
    'Environment :: Plugins',
    'Programming Language :: Python',
    'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
    'Operating System :: OS Independent',
    'Topic :: Scientific/Engineering :: Artificial Intelligence',
    'Topic :: Scientific/Engineering :: Visualization',
    'Topic :: Scientific/Engineering :: Bio-Informatics',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Intended Audience :: Education',
    'Intended Audience :: Science/Research',
    'Intended Audience :: Developers',
)

PACKAGES = find_packages(
    exclude = ('*.tests', '*.tests.*', 'tests.*', 'tests'),
)

PACKAGE_DATA = {
}

# Backwards compatibility stub. Should be removed by the 2.7 release.
PY_MODULES = ["_bioinformatics"]

SETUP_REQUIRES = (
    'setuptools',
)

INSTALL_REQUIRES = (
    'Orange',
    'setuptools',
    'numpy',
    'GenAPI>=0.0.4',
    # Dependencies which are problematic to install automatically
    #'openbabel-python', # You get bindings together with the openbabel library and not stand-alone
    #'scipy', # Requires Fortran compiler
    #'matplotlib', # Requires that numpy is installed first
),

EXTRAS_REQUIRE = {
    'GUI': (
        # Dependencies which are problematic to install automatically
        #'PyQt', # No setup.py
    ),
    'MOL_DEPICT': (
        'oasa'
    ),
    'NETWORK': (
        'Orange[NETWORK]'
    ),
    'KEGG': (
        'slumber >= 0.4'
    )

}

DEPENDENCY_LINKS = (
#    'http://bkchem.zirael.org/download/bkchem-0.13.0.tar.gz',
#    'http://orange.biolab.si/download/bkchem-0.13.0.tar.gz',
    'http://bkchem.zirael.org/download/oasa-0.13.1.tar.gz',
    'http://orange.biolab.si/download/oasa-0.13.1.tar.gz',
)

ENTRY_POINTS = {
    'orange.addons': (
        'bio = orangecontrib.bio',
    ),
    'orange.widgets': (
        'Bioinformatics = orangecontrib.bio.widgets',
        # This should be unneeded, because module given should load (register)
        # all wanted widgets and prototypes should just have a flag, but for now ...
        'Prototypes = orangecontrib.bio.widgets.prototypes',
    ),
    'orange.canvas.help': (
        'intersphinx = orangecontrib.bio.widgets:intersphinx'
    )
}

NAMESPACE_PACAKGES = ["orangecontrib", "orangecontrib.bio"]

if __name__ == '__main__':
    setup(
        name = NAME,
        version = VERSION,
        description = DESCRIPTION,
        long_description = LONG_DESCRIPTION,
        author = AUTHOR,
        author_email = AUTHOR_EMAIL,
        url = URL,
        download_url = DOWNLOAD_URL,
        license = LICENSE,
        keywords = KEYWORDS,
        classifiers = CLASSIFIERS,
        packages = PACKAGES,
        package_data = PACKAGE_DATA,
        py_modules = PY_MODULES,
        setup_requires = SETUP_REQUIRES,
        install_requires = INSTALL_REQUIRES,
        extras_require = EXTRAS_REQUIRE,
        dependency_links = DEPENDENCY_LINKS,
        entry_points = ENTRY_POINTS,
        namespace_packages=NAMESPACE_PACAKGES,
        include_package_data = True,
        zip_safe = False,
    )



import Orange.bio.kegg
import Orange.bio.gene

targets = Orange.bio.kegg.KEGGOrganism("9606").get_genes() #KEGG gene IDs

gmkegg = Orange.bio.gene.GMKEGG("9606")
gmgo = Orange.bio.gene.GMGO("9606")
gmkegggo = Orange.bio.gene.matcher([[gmkegg, gmgo]], direct=False) #joined matchers

gmkegg.set_targets(targets)
gmgo.set_targets(targets)
gmkegggo.set_targets(targets)

genes = [ "cct7", "pls1", "gdi1", "nfkb2", "a2a299" ]

print "%12s" % "gene", "%12s" % "KEGG", "%12s" % "GO", "%12s" % "KEGG+GO"
for gene in genes:
    print "%12s" % gene, "%12s" % gmkegg.umatch(gene), \
          "%12s" % gmgo.umatch(gene), \
          "%12s" % gmkegggo.umatch(gene)



###################################################################################
## numpy extension by PJ (peter.juvan@fri.uni-lj.si)
##
## TODO: fix default axis asignment (in several funcions 0 is the dedault):
##      - for numpy default is None
##      - for numpy.me default is still -1 (will this be fixed in future releases?)
###################################################################################

import numpy

###################################################################################
## pretty print
###################################################################################

def strSimMtrx(m, names, numPos=None, sort=False):
    """ print similarity matrix with fixed number of positions,
    appends names, optionally sorts names and m
    """
    assert len(names) == m.shape[0]
    assert len(m.shape) == 2
    m = numpy.asarray(m)
    if sort == True:
        nameIndList = zip(names, range(len(names)))
        nameIndList.sort()
        names = map(lambda x: x[0], nameIndList)
        sortArgs = map(lambda x: x[1], nameIndList)
        m = m[sortArgs][:,sortArgs]
    out = ""
    if numPos == None:
        numPos = max(map(lambda n: len(n), names))
    # number of decimal places
    numWhole = len(str(int(numpy.max(m))))
    numDec = numPos - numWhole - 1
    if numDec < 0:
        numDec = 0
    # print header row
    printStr = "%%s %%%is" % numPos
    out += reduce(lambda a,b: printStr % (a, b[:numPos]), names, " "*numPos)
    # print other rows
    printStrName = "%%%is" % numPos
    printStrMtrx2 = "%%.%if" % numDec
    for rowIdx,row in enumerate(m):
        rowStr = map(lambda x: printStrMtrx2 % x, row)
        out += "\n%s" % reduce(lambda a,b: printStr % (a, b[:numPos]), rowStr, printStrName % names[rowIdx][:numPos])
    return out


###################################################################################
## spearman r / pearson r between values which are present in both arrays
## uses scipy.stats
###################################################################################

def spearmanrMA(m1, m2):
    # calculate spearmanr between values which are present in both arrays
    try:
        import scipy.stats
    except ImportError:
        print "scipy.stats not found; you can download it from www.scipy.org"
    nomask12 = numpy.logical_not(numpy.logical_or(numpy.ma.getmaskarray(m1), numpy.ma.getmaskarray(m2)))
    return scipy.stats.spearmanr(m1.compress(nomask12), m2.compress(nomask12))

def pearsonrMA(m1, m2):
    # calculate pearson between values which are present in both arrays
    try:
        import scipy.stats
    except ImportError:
        print "scipy.stats not found; you can download it from www.scipy.org"
    nomask12 = numpy.logical_not(numpy.logical_or(numpy.ma.getmaskarray(m1), numpy.ma.getmaskarray(m2)))
    return scipy.stats.pearsonr(m1.compress(nomask12), m2.compress(nomask12))


###################################################################################
## argmaxMA(m,axis=-1)
## replacement for numpy.ma.argmax which does not work with masked arrays:
##  numpy.ma.argmax(numpy.ma.asarray([1,2])*numpy.ma.masked) => 0
##  numpyextn.argmax(numpy.ma.asarray([1,2])*numpy.ma.masked) => numpy.ma.masked
###################################################################################

def argmaxMA(m,axis=-1):
    return numpy.ma.where(numpy.all(numpy.ma.getmaskarray(m),axis), numpy.ma.masked, numpy.ma.argmax(m,axis))


###################################################################################
## write to tab delimited file, replace masked data by "?" by default
###################################################################################

def tofileTabMa(m, fileName, fill="?"):
    assert numpy.rank(m) == 2
    m = numpy.ma.asarray(m)
    ms = numpy.asarray(numpy.ma.filled(m), "str")
    mss = numpy.where(numpy.ma.getmaskarray(m), fill, ms)
    fd = file(fileName, "w")
    for row in mss:
        fd.write("%s\n" % reduce(lambda a,b: "%s\t%s" % (a,b), row))
    fd.close()


###################################################################################
## median
###################################################################################

def median(m,axis=0):
    """Returns median of the given nonmasked m array along the given axis.
    """
    if numpy.ma.isMA(m):
       raise TypeError, "wrong type for m (%s), try using medianMA instead" % type(m).__name__
    m = numpy.asarray(m, numpy.float)
    # do not use swapaxes: (0,1,2) -swap-> (2,1,0); (0,1,2) -transpose-> (2,0,1)
    m = numpy.transpose(m, [axis]+range(axis)+range(axis+1,numpy.rank(m)))
    return numpy.median(m)


def medianMA(m,axis=0):
    """Returns median of the given masked array m along the given axis.
    """
    return percentilesMa(m,0.5,axis)


def percentilesMa(m,perc,axis=0):
    """Returns the percentiles of the given masked array m along the given axis.
    """
    assert 0 < perc < 1
    m = numpy.ma.asarray(m, numpy.float)
    # 2008-06-23: BUG: does not work with negative axis values: 0 axis is always taken
    # assert -len(m.shape) <= axis < len(m.shape)
    assert 0 <= axis < len(m.shape)
    # 2008-06-23 BUG: at arbitrary axis, a single element is just returned
    ##    if len(m.shape) == 0 or (len(m.shape)==1 and m.shape[0]==1):
    if m.shape[axis] == 1:
        return m[axis]
    elif len(m.shape) == 1:
        # returns (1,) array or ma.masked
        mCount = numpy.ma.count(m)
        if mCount == 0:
            return numpy.ma.masked
        elif mCount == 1:
            return m.compressed()[0]
        else:
            _k = float(perc)*(mCount-1)
            k = int(_k)
            d = _k - k
            mIndSort = numpy.ma.argsort(m,fill_value=1e20)
            return m[mIndSort[k]] + d*(m[mIndSort[k+1]]-m[mIndSort[k]])
    else:
        # count the number of nonmasked indices along the given axis
        _k = float(perc) * (numpy.ma.count(m, axis=axis)-1)
        k = _k.astype(numpy.int)
        d = _k - k
        # prepare indices for other axis (except for the given)
        cind = numpy.ma.indices(numpy.ma.shape(k))
        # indices of m sorted by increasing value at axis 0; masked values placed at the end;
        mtIndSort = numpy.ma.argsort(m,axis,fill_value=1e20)
        # get indices to address median elements from m
        # tuple(takeInd1): all lists in the tuple must be of the same shape,
        #   the result mtIndSort[tuple(takeInd1)] is also of that shape
        takeInd1 = cind.tolist()
        takeInd1.insert(axis,k.tolist())
        medInd1 = mtIndSort[tuple(takeInd1)]
        takeInd2 = cind.tolist()
        takeInd2.insert(axis,(k+1).tolist())
        medInd2 = mtIndSort[tuple(takeInd2)]
        # get both median elements; if c[i] odd: med1[i] == med2[i]
        takeMed1 = cind.tolist()
        takeMed1.insert(axis,medInd1.tolist())
        med1 = m[tuple(takeMed1)]
        takeMed2 = cind.tolist()
        takeMed2.insert(axis,medInd2.tolist())
        med2 = m[tuple(takeMed2)]
##        if __name__=="__main__":
##            print "m\n",numpy.ma.filled(m,-1)
##            print "k", k
##            print "[(k-1)/2,k/2]", [(k-1)/2,k/2]
##            print "cind\n",cind
##            print "mtIndSort\n",mtIndSort
##            print "medInd1\n",medInd1
##            print "medInd2\n",medInd2
##            print "med1\n",med1
##            print "med2\n",med2
        return med1 + numpy.ma.filled(d*(med2-med1),0)

if __name__ == "__main__":
    # 2008-06-23: bug with negative indices (not fixed, added assertion
##    print "first +", medianMA(numpy.asarray([[10,1],[2,11]]),0)
##    print "first -", medianMA(numpy.asarray([[10,1],[2,11]]),-2)
##    print "second +", medianMA(numpy.asarray([[10,1],[2,11]]),1)
##    print "second -", medianMA(numpy.asarray([[10,1],[2,11]]),-1)
    # 2008-06-23: bug with medianMA(numpy.asarray([[149.0, 87.0]]), 0)
    print medianMA(numpy.asarray([149.0, 87.0]), 0)
    print medianMA(numpy.asarray([[149.0, 87.0]]), 0)
    print medianMA(numpy.asarray([[149.0, 87.0]]), 1)


## Automatically adapted for numpy.oldnumeric Oct 04, 2007 by PJ

import math
import numpy.oldnumeric as Numeric
import numpy.oldnumeric.ma as MA
import numpy.oldnumeric.mlab as MLab, numpy.oldnumeric.linear_algebra as LinearAlgebra


#####################################################################################
## LOWESS (from Biopython, extended to interpolation/extrapolation)
#####################################################################################

def lowess2(x, y, xest, f=2./3., iter=3):
    """Returns estimated values of y in data points xest (or None if estimation fails).
    Lowess smoother: Robust locally weighted regression.
    The lowess function fits a nonparametric regression curve to a scatterplot.
    The arrays x and y contain an equal number of elements; each pair
    (x[i], y[i]) defines a data point in the scatterplot. The function returns
    the estimated (smooth) values of y.

    The smoothing span is given by f. A larger value for f will result in a
    smoother curve. The number of robustifying iterations is given by iter. The
    function will run faster with a smaller number of iterations."""
    x = Numeric.asarray(x, 'd')
    y = Numeric.asarray(y, 'd')
    xest = Numeric.asarray(xest, 'd')
    n = len(x)
    nest = len(xest)
    r = min(int(Numeric.ceil(f*n)),n-1) # radius: num. of points to take into LR
    h = [Numeric.sort(abs(x-x[i]))[r] for i in range(n)]    # distance of the r-th point from x[i]
    w = Numeric.clip(abs(([x]-Numeric.transpose([x]))/h),0.0,1.0)
    w = 1-w*w*w
    w = w*w*w
    hest = [Numeric.sort(abs(x-xest[i]))[r] for i in range(nest)]    # r-th min. distance from xest[i] to x
    west = Numeric.clip(abs(([xest]-Numeric.transpose([x]))/hest),0.0,1.0)  # shape: (len(x), len(xest)
    west = 1-west*west*west
    west = west*west*west
    yest = Numeric.zeros(n,'d')
    yest2 = Numeric.zeros(nest,'d')
    delta = Numeric.ones(n,'d')
    try:
        for iteration in range(iter):
            # fit xest
            for i in range(nest):
                weights = delta * west[:,i]
                b = Numeric.array([sum(weights*y), sum(weights*y*x)])
                A = Numeric.array([[sum(weights), sum(weights*x)], [sum(weights*x), sum(weights*x*x)]])
                beta = LinearAlgebra.solve_linear_equations(A,b)
                yest2[i] = beta[0] + beta[1]*xest[i]
            # fit x (to calculate residuals and delta)
            for i in range(n):
                weights = delta * w[:,i]
                b = Numeric.array([sum(weights*y), sum(weights*y*x)])
                A = Numeric.array([[sum(weights), sum(weights*x)], [sum(weights*x), sum(weights*x*x)]])
                beta = LinearAlgebra.solve_linear_equations(A,b)
                yest[i] = beta[0] + beta[1]*x[i]
            residuals = y-yest
            s = MLab.median(abs(residuals))
            delta = Numeric.clip(residuals/(6*s),-1,1)
            delta = 1-delta*delta
            delta = delta*delta
    except LinearAlgebra.LinAlgError:
        print "Warning: NumExtn.lowess2: LinearAlgebra.solve_linear_equations: Singular matrix"
        yest2 = None
    return yest2

##x = Numeric.array([0,2.1,4.5, 6.], 'd')
##xs = Numeric.array([0,1,2,3,4,5,6], 'd')
##y = Numeric.array([0.8, 2.3, 2.9, 3.1], 'd')
##f = 2./3.
##iter=3
##import Bio.Statistics.lowess
##print Bio.Statistics.lowess.lowess(x, y, f, iter)
##print lowess2(x, y, xs, f, iter)

def lowessW(x, y, xest, f=2./3., iter=3, dWeights=None, callback=None):
    """Returns estimated values of y in data points xest (or None if estimation fails).
    Lowess smoother: Robust locally weighted regression.
    The lowess function fits a nonparametric regression curve to a scatterplot.
    The arrays x and y contain an equal number of elements; each pair
    (x[i], y[i]) defines a data point in the scatterplot. The function returns
    the estimated (smooth) values of y.

    The smoothing span is given by f. A larger value for f will result in a
    smoother curve. The number of robustifying iterations is given by iter. The
    function will run faster with a smaller number of iterations.

    Data points may be assigned weights; if None, all weights equal 1.
    """
    x = Numeric.asarray(x, 'd')
    y = Numeric.asarray(y, 'd')
    xest = Numeric.asarray(xest, 'd')
    n = len(x)
    if n <> len(y):
        raise AttributeError, "Error: lowessW(x,y,xest,f,iter,dWeights): len(x)=%i not equal to len(y)=%i" % (len(x), len(y))
    nest = len(xest)
    # weights of data points (optional)
    if dWeights <> None:
        dWeights = Numeric.asarray(dWeights, 'd')
        if len(dWeights) <> n:
            raise AttributeError, "Error: lowessW(x,y,xest,f,iter,dWeights): len(dWeights)=%i not equal to len(x)=%i" % (len(dWeights), len(x))
##        dWeights = dWeights.reshape((n,1))
    else:
##        dWeights = Numeric.ones((n,1))
        dWeights = Numeric.ones((n,))
    r = min(int(Numeric.ceil(f*n)),n-1) # radius: num. of points to take into LR
    h = [Numeric.sort(abs(x-x[i]))[r] for i in range(n)]    # distance of the r-th point from x[i]
    w = Numeric.clip(abs(([x]-Numeric.transpose([x]))/h),0.0,1.0)
    w = 1-w*w*w
    w = w*w*w
    hest = [Numeric.sort(abs(x-xest[i]))[r] for i in range(nest)]    # r-th min. distance from xest[i] to x
    west = Numeric.clip(abs(([xest]-Numeric.transpose([x]))/hest),0.0,1.0)  # shape: (len(x), len(xest))
    west = 1-west*west*west
    west = west*west*west
    yest = Numeric.zeros(n,'d')
    yest2 = Numeric.zeros(nest,'d')
    delta = Numeric.ones(n,'d')
    try:
        for iteration in range(int(iter)):
            # fit xest
            for i in range(nest):
##                print delta.shape, west[:,i].shape, dWeights.shape
                weights = delta * west[:,i] * dWeights
                b = Numeric.array([sum(weights*y), sum(weights*y*x)])
                A = Numeric.array([[sum(weights), sum(weights*x)], [sum(weights*x), sum(weights*x*x)]])
                beta = LinearAlgebra.solve_linear_equations(A,b)
                yest2[i] = beta[0] + beta[1]*xest[i]
            # fit x (to calculate residuals and delta)
            for i in range(n):
                weights = delta * w[:,i] * dWeights
                b = Numeric.array([sum(weights*y), sum(weights*y*x)])
                A = Numeric.array([[sum(weights), sum(weights*x)], [sum(weights*x), sum(weights*x*x)]])
                beta = LinearAlgebra.solve_linear_equations(A,b)
                yest[i] = beta[0] + beta[1]*x[i]
            residuals = y-yest
            s = MLab.median(abs(residuals))
            delta = Numeric.clip(residuals/(6*s),-1,1)
            delta = 1-delta*delta
            delta = delta*delta
            if callback: callback()
    except LinearAlgebra.LinAlgError:
        print "Warning: NumExtn.lowessW: LinearAlgebra.solve_linear_equations: Singular matrix"
        yest2 = None
    return yest2


##if __name__ == "__main__":
##    x1 = numpy.asarray([0,1,2,3,4,5,6,7,8,9])
##    xe1 = numpy.asarray([-1,0,1,2,3,4,5,6,7,8,9,10])
##    xe2 = numpy.asarray([4,5,6])
##    y1 = numpy.asarray([0.1, 0.9, 2.05, 3.11, 3.99, 4.95, 5.88, 6.5, 6.8, 7])
##    w1 = numpy.asarray([1,   0.1,1,    1,    0.1, 0.1, 0.1, 0.1,0.1,0.1])
##
##    l2 = lowess2(x1, y1, xe1, f=0.3, iter=3)
##    print l2
##    l3 = lowessW(x1, y1, xe2, f=0.3, iter=3, dWeights=w1)
##    print l3 #, "\n", w3, "\n", west3



#####################################################################################
#####################################################################################

def denary2Binary(n):
    '''convert denary integer n to binary string bStr'''
    bStr = ''
    if n < 0:  raise ValueError, "must be a positive integer"
    if n == 0: return '0'
    while n > 0:
        bStr = str(n % 2) + bStr
        n = n >> 1
    return bStr


#####################################################################################
## position2index:
##  given the position of an element in a flat array
##  get the indices of that element in an array of given shape
##
## getIndices(m,val):
##  Input: m=[[1,2],[2,3]], val=2
##  Output: Numeric.array([(0,1),(1,0)])
#####################################################################################

def position2index(n, shape):
    """Given a position in a flat array (n), returns indices in an array of given shape.
    """
    raise DeprecationWarning, "Depricated, use positions2indices(pos, shape) instead."
    if n < 0:  raise ValueError, "must be a positive integer"
    shapeL = list(shape)
    shapeL.reverse()
    ind = []
    for shp in shapeL:
        ind = [n % shp] + ind
        n = int(n / shp)
    return ind


def positions2indices(pos, shape):
    """Given postions and shape, returns indices shaped (numPositions, len(shape));
    e.g.: pos=[0,2,8], shape=(3,3) ->  [[0,0],[2,0],[2,2]]
    """
    lenShape = len(shape)
    pos = numpy.asarray(pos)
    if pos.shape[0] == 0:
        return numpy.empty([0] + list(shape[1:]))
    posMax = numpy.max(pos)
    assert posMax < numpy.multiply.reduce(shape), "Error: Position too large for a given shape."
    assert numpy.min(pos) >= 0, "Error, position cannot be negative."
    ind = numpy.ones((pos.shape[0], lenShape))
    for dim in range(lenShape-1,-1,-1):
        ind[:,dim] = numpy.mod(pos, shape[dim])
        pos = numpy.divide(pos, shape[dim])
    return numpy.asarray(ind, numpy.int)


def indices2positions(ind, shape):
    """Given indices shaped (numPositions, len(shape)), returns corresponding positions in a falt array; output shape = (ind.shape[0],);
    e.g.: ind=[[0,0],[2,0],[2,2]] , shape=(3,3) -> [0,6,8]
    """
    ind = numpy.asarray(ind)
    if ind.shape[0] == 0:
        return numpy.empty((0,))
    assert len(ind.shape) == 2 and ind.shape[1] == len(shape), "Error: ind should be 2D array, ind.shape[1] should match len(shape)."
    assert numpy.less(numpy.max(ind,0),numpy.asarray(shape)).all(), "Error: indices do not fit the shape."
    pos = numpy.zeros((ind.shape[0],))
    for i,shp in enumerate(shape):
        pos += ind[:,i]*numpy.multiply.reduce(shape[i+1:])
    return numpy.asarray(pos,numpy.int)
    

def getPositions(m, val):
    """Input: arbitrary (masked) array and a value from that array;
    Output: array of positions of the given value in a flat m;
    """
    m = MA.asarray(m)
    return Numeric.compress(MA.equal(MA.ravel(m),val), Numeric.arange(Numeric.multiply.reduce(m.shape)))

def getIndices(m, val):
    """Input: arbitrary (masked) array and a value from that array;
    Output: array of indices corresponding to positions of the given value in array m;
    Output shape: (numb. of val values in m, len(m.shape)).
    """
    posFlat = getPositions(m, val)
    return positions2indices(posFlat, m.shape)


#####################################################################################
## Indices <-> Condition
#####################################################################################

def condition2indices(condition):
    """Input: condition=[1,0,0,1]; output: indices=[0,3]
    """
    condition = Numeric.asarray(condition)
    assert len(condition.shape) == 1
    return Numeric.compress(condition, Numeric.arange(condition.shape[0]))


def indices2condition(indices, length):
    """Input: indices=[0,3]; output: condition=[1,0,0,1]
    """
    indices = Numeric.asarray(indices)
    assert len(indices.shape) == 1
    assert length >= indices.shape[0]
    c = Numeric.zeros((length,), Numeric.Int)
    Numeric.put(c, indices, 1)
    return c

#####################################################################################
## Inverse permutation
##  permutInverse([0,2,3,5,4,1]) -> [0,5,1,2,4,3,]
##  permutInverse([0,5,1,2,4,3,]) -> [0,2,3,5,4,1]
#####################################################################################

#from Permutation import *

def permutInverse(n):
    """Returns inverse permutation given integers in range(len(n)),
    such that permitInverse(permutInverse(range(4)))==range(4).
    """
    n = Numeric.asarray(n)
    pInv = Numeric.argsort(n)
    assert Numeric.all(Numeric.equal(n, Numeric.argsort(pInv))), "Inverse not successful; input should be permutation of range(len(input))."
    return pInv

###################################################################################################################################
## Generator of random matrix of permutations
## where each row and each column represents a permutation of n elements.
###################################################################################################################################

def getRandomMtrxPermut(n):
    """Generator of random matrix of permutations;
    input: number of elements;
    returns a matrix where each row and each column represents a permutation of n elements.
    """
    mx = numpy.zeros((n,n), numpy.int)
    pArr = numpy.random.permutation(n)
    for i in range(n):
        mx[i] = numpy.concatenate((pArr[i:], pArr[:i]))
    return numpy.take(mx,numpy.random.permutation(n),0)   


#####################################################################################
## Ranks:
##      - Numeric (instead of statc.rankdata), in range 1 ... shape[0]
##      - MA: masked elements are not ranked; range: 1 ... #non-masked_values
#####################################################################################

def rankData(n, inverse=False):
    """Returns ranks of 1D Numeric array in range 1...shape[0].
    """
    n = Numeric.asarray(n)
    assert Numeric.rank(n) == 1
    r = Numeric.zeros(n.shape[0], Numeric.Float)
    Numeric.put(r, Numeric.argsort(n), Numeric.arange(n.shape[0]))
    if inverse:
        return -1*r+n.shape[0]
    else:
        return r+1

def rankDataMA(m, inverse=False):
    """Returns ranks of 1D masked array; masked values ignored, range 1...#non-masked_values.
    """
    m = MA.asarray(m)
    assert MA.rank(m) == 1
    fill_val = m.fill_value()
    m.set_fill_value(MA.maximum(m) + 1)
    r = MA.zeros(m.shape[0], Numeric.Float)
    MA.put(r, MA.argsort(m), Numeric.arange(m.shape[0]))
    m.set_fill_value(fill_val)
    r = MA.array(r, mask=MA.getmaskarray(m))
    if inverse:
        return -1*r+MA.count(m)
    else:
        return r+1
    


#####################################################################################
## Unary functions: a mask is set only in places where both a arrays are masked.
##  e.g.:
##    minus_unary(10,--)  -> 10
##    minus_unary(--,--) -> --
#####################################################################################

def subtract_unary(a, b):
    """Returns a-b with masked values only in places where both a and b are masked.
    """
    a = MA.asarray(a)
    b = MA.asarray(b)
    el = MA.subtract(a.filled(0), b.filled(0))
    mask = Numeric.logical_and(MA.getmaskarray(a), MA.getmaskarray(b))
    return MA.array(el, mask=mask)

def add_unary(a, b):
    """Returns a+b with masked values only in places where both a and b are masked.
    """
    a = MA.asarray(a)
    b = MA.asarray(b)
    el = MA.add(a.filled(0), b.filled(0))
    mask = Numeric.logical_and(MA.getmaskarray(a), MA.getmaskarray(b))
    return MA.array(el, mask=mask)

def multiply_unary(a, b):
    """Returns a*b with masked values only in places where both a and b are masked.
    """
    a = MA.asarray(a)
    b = MA.asarray(b)
    el = MA.multiply(a.filled(1), b.filled(1))
    mask = Numeric.logical_and(MA.getmaskarray(a), MA.getmaskarray(b))
    return MA.array(el, mask=mask)

def divide_unary(a, b):
    """Returns a*b with masked values only in places where both a and b are masked.
    """
    a = MA.asarray(a)
    b = MA.asarray(b)
    el = MA.divide(a.filled(1), b.filled(1))
    mask = Numeric.logical_and(MA.getmaskarray(a), MA.getmaskarray(b))
    return MA.array(el, mask=mask)


#####################################################################################
## Logical OR / AND of two masked arrays where:
##  - the unary OR equals to the element: OR(0) == 0, OR(1) == 1
##  - the mask equals to logical AND of the corresponding masks
##
## For example:
##  m1 = MA.array([0,0,0,-,-,-,1,1,1], mask=[0,0,0,1,1,1,0,0,0])
##  m2 = MA.array([0,-,1,0,-,1,0,-,1], mask=[0,1,0,0,1,0,0,1,0])
##
## logical_unary_or(m1,m2) =
##       MA.array([0,0,1,0,-,1,1,1,1], mask=[0,0,0,0,1,0,0,0,0])
## The built-in logical_or returns:
##       MA.array([0,-,0,-,-,-,0,-,1], mask=[0,1,0,1,1,1,0,1,0])
##
## logical_unary_and(m1,m2) =
##       MA.array([0,0,0,0,-,1,0,1,1], mask=[0,0,0,0,1,0,0,0,0])
## The built-in logical_and returns:
##       MA.array([0,-,0,-,-,-,0,-,1], mask=[0,1,0,1,1,1,0,1,0])
#####################################################################################

def logical_unary_or(m1, m2):
    el = Numeric.logical_or(m1.filled(0), m2.filled(0))
    mask = Numeric.logical_and(MA.getmaskarray(m1), MA.getmaskarray(m2))
    return MA.array(el, mask=mask)    


def logical_unary_and(m1, m2):
    el = Numeric.logical_and(m1.filled(1), m2.filled(1))
    mask = Numeric.logical_and(MA.getmaskarray(m1), MA.getmaskarray(m2))
    return MA.array(el, mask=mask)    


#####################################################################################
## MA.dot fix:
##  - MA.dot returns does not set the masked values (it places 0 instead)
##  - MA.dot(x,y)[i,j] == MA.masked iff MA.add.reduce(x[i,:]*y[:,j]) == MA.masked
##  - done faste enugh - it requires two matrix multiplications
#####################################################################################

def dotMA(a, b):
    """Returns dot-product for MA arrays; fixed masked values.
    """
    a = MA.asarray(a)
    b = MA.asarray(b)
    ab = MA.dot(a,b)
    # fix masked values in ab (MA.dot returns 0 instead of MA.masked)
    nonMasked = Numeric.dot(1-MA.getmaskarray(a).astype(Numeric.Int), 1-MA.getmaskarray(b).astype(Numeric.Int))
    return MA.where(Numeric.equal(nonMasked,0), MA.masked, ab)

#####################################################################################
## Apply a binary function on a sequence of arrays
#####################################################################################

def apply_binary(aFunction, *args):
    assert len(args) > 1, "at least two arguments required"
    return reduce(lambda x,y: aFunction(x,y), list(args[1:]), args[0])
##    return reduce(lambda x,y: aFunction(x,y), list(args))
        

#####################################################################################
## Strictly upper (lower) triangular matrix <-> 1D array
#####################################################################################

def triangularPut(m1d, upper=1, lower=0):
    """Returns 2D masked array with elements of the given 1D array in the strictly upper (lower) triangle.
    Elements of the 1D array should be ordered according to the upper triangular part of the 2D matrix.
    The lower triangular part (if requested) equals to the transposed upper triangular part.
    If upper == lower == 1 a symetric matrix is returned.
    """
    assert upper in [0,1] and lower in [0,1], "[0|1] expected for upper / lower"
    m1d = MA.asarray(m1d)
    assert MA.rank(m1d) == 1, "1D masked array expected"
    m2dShape0 = math.ceil(math.sqrt(2*m1d.shape[0]))
    assert m1d.shape[0] == m2dShape0*(m2dShape0-1)/2, "the length of m1d does not correspond to n(n-1)/2"
    if upper:
        if lower:
            mask = Numeric.fromfunction(lambda i,j: i==j, (m2dShape0, m2dShape0))
        else:
            mask = Numeric.fromfunction(lambda i,j: i>=j, (m2dShape0, m2dShape0))
    else:
        if lower:
            mask = Numeric.fromfunction(lambda i,j: i<=j, (m2dShape0, m2dShape0))
        else:
            mask = Numeric.ones((m2dShape0, m2dShape0))

    m2d = MA.ravel(MA.zeros((m2dShape0, m2dShape0), m1d.dtype.char))
    condUpperTriang = Numeric.fromfunction(lambda i,j: i<j, (m2dShape0, m2dShape0))
    putIndices = Numeric.compress(Numeric.ravel(condUpperTriang), Numeric.arange(0, m2dShape0**2, typecode=Numeric.Int))
    MA.put(m2d, putIndices, m1d)
    m2d = MA.reshape(m2d, (m2dShape0, m2dShape0))
    m2d = MA.where(condUpperTriang, m2d, MA.transpose(m2d))
    return MA.array(m2d, mask=Numeric.logical_or(mask, MA.getmaskarray(m2d)))


def triangularGet(m2d, upper=1):
    """Returns 1D masked array with elements from the upper (lower) triangular part of the given matrix.
    For a symetric matrix triangularGet(m2d, 0) and triangularGet(m2d, 1) return elements in different order.
    """
    assert upper in [0,1], "upper: [0|1] expected"
    m2d = MA.asarray(m2d)
    assert MA.rank(m2d) == 2, "2D (masked) array expected"
    if upper:
        takeInd = Numeric.compress(Numeric.ravel(Numeric.fromfunction(lambda i,j: i<j, m2d.shape)), Numeric.arange(0, Numeric.multiply.reduce(m2d.shape), typecode=Numeric.Int))
    else:
        takeInd = Numeric.compress(Numeric.ravel(Numeric.fromfunction(lambda i,j: i>j, m2d.shape)), Numeric.arange(0, Numeric.multiply.reduce(m2d.shape), typecode=Numeric.Int))
    return MA.ravel(m2d).take(takeInd)


#####################################################################################
## Put 1D array on the diagonal of the given 2D array and returns a copy.
#####################################################################################

def diagonalPut(m1d, m2d):
    """Puts the given 1D masked array into the diagonal of the given 2D masked array and returns a new copy of the 2D array.
    """
    m1d = MA.asarray(m1d)
    m2d = MA.asarray(m2d)
    assert MA.rank(m1d) == 1 and MA.rank(m2d) == 2, "1D and 2D masked array expected"
    assert m1d.shape[0] == m2d.shape[0] == m2d.shape[1], "the shape of the given arrays does not match"
    putIndices = Numeric.compress(Numeric.ravel(Numeric.fromfunction(lambda i,j: i==j, m2d.shape)), Numeric.arange(0, Numeric.multiply.reduce(m2d.shape), typecode=Numeric.Int))
    m2dShape = m2d.shape
    m2d = MA.ravel(m2d)
    MA.put(m2d, putIndices, m1d)
    return MA.reshape(m2d, m2dShape)
    

#####################################################################################
#### linear interpolation of missing values by the given axis
#####################################################################################
##
##def fillLinear1(ma, axis=0):
##    """BUG: does not handle successive missing values
##    """
##    maflat = MA.array(MA.ravel(ma))
##    mod = ma.shape[axis]
##    maskInd = Numeric.compress(MA.getmaskarray(maflat), Numeric.arange(maflat.shape[0]))
##    for idx in maskInd:
##        if (idx % mod) == 0:  # the first one
##            maflat[idx] = maflat[idx+1]
##        elif (idx % mod) == mod-1:
##            maflat[idx] = maflat[idx-1]
##        else:
##            maflat[idx] = (maflat[idx-1] + maflat[idx+1]) / 2.
##    return MA.reshape(maflat, ma.shape)
##
##
##def fillLinear2(ma, axis=0):
##    """BUG: does not handle successive missing values
##    """
##    maflat = MA.array(MA.ravel(ma))
##    mod = ma.shape[axis]
##    maskInd = Numeric.compress(MA.getmaskarray(maflat), Numeric.arange(maflat.shape[0]))
##    for idx in maskInd:
##        idxFirst = idx-(idx%mod)
##        idxNext = idxFirst+mod
##        if MA.count(maflat[idxFirst:idxNext]) == 0:                                 # all elements are masked
##            print "Warning: cannot do linear interpolation, all values are masked"
##        elif idx == idxFirst:                                                        # the first element is masked
##            maflat[idx] = maflat[idx+1:idxNext].compressed()[0]
##        elif idx == idxNext-1:                                                      # the last element is masked
##            maflat[idx] = maflat[idxFirst:idx].compressed()[-1]
##        else:
##            maflat[idx] = MA.average([maflat.take(range(idx-1,idxFirst-1,-1)).compressed(), maflat[idx+1:idxNext].compressed(), MA.masked])
####            maflat[idx] = (maflat[idxFirst:idx].compressed()[-1] + maflat[idx+1:idxNext].compressed()[0]) / 2.
##    return MA.reshape(maflat, ma.shape)
##
##def fillLinear(ma, axis=0):
##    """Linear interpolation of missing values;
##    """
##    pass
##
##
##si = scipy.interpolate.interp1d(Numeric.array([[0,1,3,4],[0,1,3,4]]), Numeric.array([[0.1,1.1,3.2,4.4],[0,1,3,4]]))
##si(2)

###################################################################################
## swapaxes for masked array
###################################################################################

def swapaxesMA(ma, axis1, axis2):
    if axis1 < axis2:
        m,M = axis1, axis2
    elif axis1 > axis2:
        m,M = axis2, axis1
    elif 0 <= axis1 == axis2 < len(ma.shape):
        return ma
    else:
        raise ValueError("Bad axis1 or axis2 argument to swapaxes.")
    axList = range(m) + [M] + range(m+1,M) + [m] + range(M+1,len(ma.shape))
    return MA.transpose(ma, axList)


###################################################################################
## compress MA to Numeric and return Numeric + indices of non-masked places
###################################################################################

def compressIndices(ma):
    """Returns 1D compressed Numeric array and the indices of the non-masked places.
    usage:  nu,ind = compressIndices(ma)
            nu = Numeric.elementwise_function(nu)
            ma = MA.put(ma, ind, nu)
    """
    ma = MA.asarray(ma)
    nonMaskedInd = Numeric.compress(1-Numeric.ravel(MA.getmaskarray(ma)), Numeric.arange(Numeric.multiply.reduce(ma.shape)))
    return MA.filled(ma.compressed()), nonMaskedInd


###################################################################################
## compact print of Numeric or MA
###################################################################################

def aprint(a,decimals=2):
    """prints array / masked array of floats"""
    if isinstance(a, Numeric.ArrayType):
        print Numeric.around(a.astype(Numeric.Float0),decimals)
    elif type(a) == MA.MaskedArray:
        print MA.around(a.astype(Numeric.Float0),decimals)


###################################################################################
## min / max along the given axis
## SLOW DUE TO SORT
###################################################################################

def minMA(m,axis=0):
    """slow: remove sorting"""
    m = MA.asarray(m, Numeric.Float)
    transList = [axis] + range(0,axis) + range(axis+1,MA.rank(m))
    m = MA.transpose(m, transList)    # do not use swapaxes: (0,1,2) -swap-> (2,1,0); (0,1,2) -transpose-> (2,0,1)
    return MA.sort(m, 0, fill_value=1e20)[0]
    
def maxMA(m,axis=0):
    """slow: remove sorting"""
    m = MA.asarray(m, Numeric.Float)
    transList = [axis] + range(0,axis) + range(axis+1,MA.rank(m))
    m = MA.transpose(m, transList)    # do not use swapaxes: (0,1,2) -swap-> (2,1,0); (0,1,2) -transpose-> (2,0,1)
    return MA.sort(m, 0, fill_value=-1e20)[-1]


###################################################################################
## median absolute deviation
###################################################################################

def mad(m,axis=0):
    """Returns Median Absolute Deviation of the given array along the given axis.
    """
    m = Numeric.asarray(m)
    mx = Numeric.asarray(median(m,axis),Numeric.Float)
    xt = Numeric.transpose(m, [axis]+range(axis)+range(axis+1,Numeric.rank(m))) # do not use swapaxes: (0,1,2) -swap-> (2,1,0); (0,1,2) -transpose-> (2,0,1)
    return MLab.median(Numeric.absolute(xt-mx))

def madMA(m,axis=0):
    """Returns Median Absolute Deviation of the given masked array along the given axis.
    """
    m = MA.asarray(m)
    mx = MA.asarray(medianMA(m,axis),Numeric.Float)
    xt = MA.transpose(m, [axis]+range(axis)+range(axis+1,MA.rank(m)))   # do not use swapaxes: (0,1,2) -swap-> (2,1,0); (0,1,2) -transpose-> (2,0,1)
    return medianMA(MA.absolute(xt-mx))



if __name__ == "__main__":
##    n2 = numpy.asarray([[1,2],[3,4],[5,6]], numpy.float)    #shape (3,2)
##    n3 = numpy.asarray([n2, 2*n2, 3*n2, 4*n2])              #shape (4,3,2)
##
##    m2 = numpy.ma.asarray(n2)
##    m2[0,0] = numpy.ma.masked
##    m2[1,1] = numpy.ma.masked
##
##    m3 = numpy.ma.asarray(n3)
##    m3[0,0,0] = 10000
##    m3[1,1,1] = 10000
##    m3[2,2,0] = 10000
##    m3[3,0,1] = 10000
##    m3[0,0,0] = numpy.ma.masked
##    m3[1,1,1] = numpy.ma.masked
##    m3[2,2,0] = numpy.ma.masked
##    m3[3,0,1] = numpy.ma.masked
##
##    mm3 = numpy.ma.ones((4,3,2), numpy.float) * numpy.ma.masked
##    mm3[0,0,0] = 33
##    mm3[1,1,1] = 333
##    mm3[2,2,0] = 3333
##    mm3[3,0,1] = 33333
##
##    # compare to Numeric
##    import Numeric, MA
##    n2n = Numeric.asarray([[1,2],[3,4],[5,6]], Numeric.Float)    #shape (3,2)
##    n3n = Numeric.asarray([n2n, 2*n2n, 3*n2n, 4*n2n])              #shape (4,3,2)
##
##    m2n = MA.asarray(n2n)
##    m2n[0,0] = MA.masked
##    m2n[1,1] = MA.masked
##
##    m3n = MA.asarray(n3n)
##    m3n[0,0,0] = 10000
##    m3n[1,1,1] = 10000
##    m3n[2,2,0] = 10000
##    m3n[3,0,1] = 10000
##    m3n[0,0,0] = MA.masked
##    m3n[1,1,1] = MA.masked
##    m3n[2,2,0] = MA.masked
##    m3n[3,0,1] = MA.masked
##
##    mm3n = MA.ones((4,3,2), Numeric.Float) * MA.masked
##    mm3n[0,0,0] = 33
##    mm3n[1,1,1] = 333
##    mm3n[2,2,0] = 3333
##    mm3n[3,0,1] = 33333

    # argmaxMA    
    m2 = numpy.ma.asarray([[1,2,3],[4,5,6]])*numpy.ma.masked
    m2[1,2] = 10    
##    print "orig, axis None\t", numpy.ma.argmax(m2, None)
##    print "extn, axis None\t", argmaxMA(m2, None)
##    print "orig, axis 0\t", numpy.ma.argmax(m2,0)
##    print "extn, axis 0\t", argmaxMA(m2,0)
##    print "orig, axis 1\t", numpy.ma.argmax(m2,1)
##    print "extn, axis 1\t", argmaxMA(m2,1)
    ma = numpy.ma.argmax(m2,1)
    mb = argmaxMA(m2,1)



"""
Bioinformatics
==============

"""

NAME = "Bioinformatics"

ICON = "icons/Category-Bioinformatics.svg"

BACKGROUND = "light-grass"

intersphinx = (
    ("{DEVELOP_ROOT}/docs/build/html/", None),
    ("http://orange-bioinformatics.readthedocs.org/en/latest/", None)
)
